@@ -1,8 +1,26 @@
-import enum
-from typing import NoReturn, Optional
+# PyFLP - An FL Studio project file (.flp) parser
+# Copyright (C) 2022 demberto
+#
+# This program is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option)
+# any later version. This program is distributed in the hope that it will be
+# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
+# Public License for more details. You should have received a copy of the
+# GNU General Public License along with this program. If not, see
+# <https://www.gnu.org/licenses/>.
 
-from pyflp.flobject import _FLObject
-from pyflp.properties import _UIntProperty, _EnumProperty, _FloatProperty, _BoolProperty
+import enum
+from typing import Optional
+
+from pyflp._flobject import _FLObject
+from pyflp._properties import (
+    _BoolProperty,
+    _EnumProperty,
+    _FloatProperty,
+    _UIntProperty,
+)
 
 
 class ChannelArp(_FLObject):
@@ -23,7 +41,6 @@ class ChannelArp(_FLObject):
 
     # * Properties
     direction: Direction = _EnumProperty(Direction)
-    """See `Direction`."""
 
     range: Optional[int] = _UIntProperty()
     """Range (in octaves)."""
@@ -42,7 +59,3 @@ class ChannelArp(_FLObject):
 
     slide: Optional[bool] = _BoolProperty()
     """Whether arpeggio will slide between notes."""
-
-    def _save(self) -> NoReturn:
-        """Implemented in `ChannelParametersEvent`."""
-        raise NotImplementedError
