@@ -1,9 +1,39 @@
-import enum
-import colour
-import struct
-from typing import List, Optional, Dict, Tuple
+# PyFLP - An FL Studio project file (.flp) parser
+# Copyright (C) 2022 demberto
+#
+# This program is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option)
+# any later version. This program is distributed in the hope that it will be
+# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
+# Public License for more details. You should have received a copy of the
+# GNU General Public License along with this program. If not, see
+# <https://www.gnu.org/licenses/>.
 
-from pyflp.constants import WORD, DWORD, TEXT, DATA
+import enum
+import struct
+from typing import Dict, List, Optional, Tuple
+
+import colour
+
+from pyflp._event import (
+    _ByteEvent,
+    _DataEventType,
+    _DWordEventType,
+    _EventType,
+    _TextEvent,
+    _WordEvent,
+)
+from pyflp._flobject import _FLObject
+from pyflp._properties import (
+    _BoolProperty,
+    _ColorProperty,
+    _EnumProperty,
+    _IntProperty,
+    _StrProperty,
+    _UIntProperty,
+)
 from pyflp.channel.arp import ChannelArp
 from pyflp.channel.delay import ChannelDelay
 from pyflp.channel.envlfo import (
@@ -16,28 +46,11 @@ from pyflp.channel.level_offsets import ChannelLevelOffsets
 from pyflp.channel.levels import ChannelLevels
 from pyflp.channel.polyphony import ChannelPolyphony
 from pyflp.channel.tracking import ChannelTracking, ChannelTrackingEvent
-from pyflp.event import (
-    _EventType,
-    ByteEvent,
-    WordEvent,
-    _DWordEventType,
-    _DataEventType,
-    TextEvent,
-)
-from pyflp.flobject import _FLObject
+from pyflp.constants import DATA, DWORD, TEXT, WORD
 from pyflp.insert.insert import Insert
-from pyflp.plugin.plugin import _Plugin
+from pyflp.plugin._plugin import _Plugin
 from pyflp.plugin.synths.boobass import BooBass
 from pyflp.plugin.vst import VSTPlugin
-from pyflp.properties import (
-    _UIntProperty,
-    _EnumProperty,
-    _BoolProperty,
-    _IntProperty,
-    _StrProperty,
-    _ColorProperty,
-)
-from pyflp.validators import _UIntValidator, _IntValidator
 
 
 class Channel(_FLObject):
@@ -217,15 +230,15 @@ class Channel(_FLObject):
 
     index: int = _UIntProperty()
 
-    volume: Optional[int] = _UIntProperty(_UIntValidator(12800))
+    volume: Optional[int] = _UIntProperty(max_=12800)
     """Min: 0, Max: 12800, Default: 10000."""
 
-    pan: Optional[int] = _UIntProperty(_UIntValidator(12800))
+    pan: Optional[int] = _UIntProperty(max_=12800)
     """Min: 0, Max: 12800, Default: 6400."""
 
     color: colour.Color = _ColorProperty()
 
-    target_insert: Optional[int] = _IntProperty(_IntValidator(-1, Insert.max_count))
+    target_insert: Optional[int] = _IntProperty(min_=-1, max_=Insert.max_count)
     """The index of the `Insert` the channel is routed to."""
 
     kind: Optional[Kind] = _EnumProperty(Kind)
@@ -245,7 +258,8 @@ class Channel(_FLObject):
 
     @zipped.setter
     def zipped(self, value: bool):
-        assert isinstance(value, bool)
+        if not isinstance(value, bool):
+            raise TypeError
         self._setprop("zipped", value)
 
     root_note: Optional[int] = _IntProperty()
@@ -358,7 +372,8 @@ class Channel(_FLObject):
 
     @cut_group.setter
     def cut_group(self, value: Tuple[int]):
-        assert len(value) == 2
+        if len(value) != 2:
+            raise TypeError("Expected a tuple of size 2")
         self._events["cut_group"].data = struct.pack("2H", *value)
         self._cut_group = value
 
@@ -370,7 +385,7 @@ class Channel(_FLObject):
             return self._fx.parse_event(e)
         return super().parse_event(e)
 
-    def _parse_byte_event(self, e: ByteEvent):
+    def _parse_byte_event(self, e: _ByteEvent):
         if e.id == Channel.EventID.Enabled:
             self._parse_bool(e, "enabled")
         elif e.id == Channel.EventID._Vol:
@@ -393,7 +408,7 @@ class Channel(_FLObject):
         elif e.id == Channel.EventID.Locked:
             self._parse_bool(e, "locked")
 
-    def _parse_word_event(self, e: WordEvent):
+    def _parse_word_event(self, e: _WordEvent):
         if e.id == Channel.EventID.New:
             self._parse_H(e, "index")
         elif e.id == Channel.EventID._Volume:
@@ -427,7 +442,7 @@ class Channel(_FLObject):
         elif e.id == Channel.EventID.AUSampleRate:
             self._parse_I(e, "au_sample_rate")
 
-    def _parse_text_event(self, e: TextEvent):
+    def _parse_text_event(self, e: _TextEvent):
         if e.id == Channel.EventID.DefaultName:
             self._parse_s(e, "default_name")
         elif e.id == Channel.EventID.SamplePath:
