@@ -1,62 +1,89 @@
+# PyFLP - An FL Studio project file (.flp) parser
+# Copyright (C) 2022 demberto
+#
+# This program is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option)
+# any later version. This program is distributed in the hope that it will be
+# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
+# Public License for more details. You should have received a copy of the
+# GNU General Public License along with this program. If not, see
+# <https://www.gnu.org/licenses/>.
+
+import enum
 import io
 import zipfile
 from pathlib import Path
 from typing import List, Set, Union
 
 from bytesioex import BytesIOEx
-from pyflp.controllers import RemoteController, RemoteControllerEvent
 
-from pyflp.flobject import _FLObject
+from pyflp._event import (
+    _ByteEvent,
+    _ColorEvent,
+    _DataEvent,
+    _DWordEvent,
+    _Event,
+    _EventType,
+    _TextEvent,
+    _WordEvent,
+)
+from pyflp._flobject import _FLObject
 from pyflp.arrangement.arrangement import Arrangement
 from pyflp.arrangement.playlist import Playlist
 from pyflp.arrangement.timemarker import TimeMarker
 from pyflp.arrangement.track import Track, TrackDataEvent
-from pyflp.misc import Misc
 from pyflp.channel.channel import Channel
 from pyflp.channel.delay import ChannelDelayEvent
 from pyflp.channel.envlfo import ChannelEnvelopeLFOEvent
 from pyflp.channel.filter import Filter
 from pyflp.channel.fx import ChannelFX
-from pyflp.channel.levels import ChannelLevelsEvent
 from pyflp.channel.level_offsets import ChannelLevelOffsetsEvent
+from pyflp.channel.levels import ChannelLevelsEvent
 from pyflp.channel.parameters import ChannelParametersEvent
 from pyflp.channel.polyphony import ChannelPolyphonyEvent
 from pyflp.channel.tracking import ChannelTrackingEvent
-from pyflp.constants import BYTE, DATA, DATA_TEXT_EVENTS, DWORD, TEXT, WORD
-from pyflp.event import (
-    Event,
-    _EventType,
-    ByteEvent,
-    WordEvent,
-    DataEvent,
-    DWordEvent,
-    ColorEvent,
-    TextEvent,
+from pyflp.constants import (
+    BYTE,
+    DATA,
+    DATA_MAGIC,
+    DATA_TEXT_EVENTS,
+    DWORD,
+    HEADER_MAGIC,
+    HEADER_SIZE,
+    TEXT,
+    WORD,
 )
+from pyflp.controllers import RemoteController, RemoteControllerEvent
+from pyflp.exceptions import InvalidHeaderSizeError, InvalidMagicError
 from pyflp.insert.event import InsertParamsEvent
 from pyflp.insert.insert import Insert
-from pyflp.insert.slot import InsertSlot
 from pyflp.insert.parameters import InsertParametersEvent
-from pyflp.pattern.pattern import Pattern
+from pyflp.insert.slot import InsertSlot
+from pyflp.misc import Misc
 from pyflp.pattern.controller import PatternControllersEvent
 from pyflp.pattern.note import PatternNotesEvent
+from pyflp.pattern.pattern import Pattern
 from pyflp.plugin.vst import VSTPluginEvent
 from pyflp.project import Project
 from pyflp.utils import FLVersion
 
+__all__ = ["Parser"]
+
 
 class Parser:
     """FL Studio project file parser."""
 
-    __INSERT_EVENTS = []
+    __INSERT_EVENTS: List[enum.IntEnum] = []
     for en in (Insert.EventID, InsertSlot.EventID):
         __INSERT_EVENTS.extend(en.__members__.values())
 
-    __ARRANGEMENT_EVENTS = []
+    __ARRANGEMENT_EVENTS: List[enum.IntEnum] = []
     for en in (Arrangement.EventID, Track.EventID, Playlist.EventID):
         __ARRANGEMENT_EVENTS.extend(en.__members__.values())
 
-    __CHANNEL_EVENTS = []
+    __CHANNEL_EVENTS: List[enum.IntEnum] = []
     for en in (Channel.EventID, ChannelFX.EventID):
         __CHANNEL_EVENTS.extend(en.__members__.values())
 
@@ -80,7 +107,7 @@ class Parser:
         # See `__parse_pattern` below for details.
         self.__pat_indexes: Set[int] = set()
 
-        # If current plugin defautl name is "Fruity Wrapper", this is True.
+        # If current plugin default name is "Fruity Wrapper", this is True.
         # `__build_event_store` will instantiate `VSTPluginEvent`.
         self.__cur_plug_is_vst = False
 
@@ -91,7 +118,7 @@ class Parser:
         USING MULTIPLE INSTANCES AT THE SAME TIME."""
 
         _FLObject._count = 0
-        Event._count = 0
+        _Event._count = 0
         for subclass in _FLObject.__subclasses__():
             subclass._count = 0
 
@@ -100,17 +127,23 @@ class Parser:
 
         def add_dwordevent(id, buf):
             if id in self.__COLOR_EVENTS:
-                ev = ColorEvent(id, buf)
+                ev = _ColorEvent(id, buf)
             else:
-                ev = DWordEvent(id, buf)
+                ev = _DWordEvent(id, buf)
             self.__events.append(ev)
 
         def add_textevent(id, buf):
             if id == Misc.EventID.Version:
-                _FLObject.fl_version = flv = FLVersion(TextEvent.as_ascii(buf))
+                _FLObject._fl_version = flv = FLVersion(_TextEvent.as_ascii(buf))
                 if flv.as_float() < 11.5:
-                    TextEvent.uses_unicode = False
-            self.__events.append(TextEvent(id, buf))
+                    _TextEvent.uses_unicode = False
+            ev = _TextEvent(id, buf)
+            if (
+                id in (InsertSlot.EventID.DefaultName, Channel.EventID.DefaultName)
+                and ev.to_str() == "Fruity Wrapper"
+            ):
+                self.__cur_plug_is_vst = True
+            self.__events.append(ev)
 
         def add_dataevent(id, buf):
             if id == Track.EventID.Data:
@@ -132,8 +165,9 @@ class Parser:
             elif id in (Channel.EventID.Plugin, InsertSlot.EventID.Plugin):
                 if self.__cur_plug_is_vst:
                     ev = VSTPluginEvent(id, buf)
+                    self.__cur_plug_is_vst = False
                 else:
-                    ev = DataEvent(id, buf)
+                    ev = _DataEvent(id, buf)
             elif id == Insert.EventID.Parameters:
                 ev = InsertParametersEvent(buf)
             elif id == Pattern.EventID.Controllers:
@@ -143,7 +177,7 @@ class Parser:
             elif id == RemoteController.ID:
                 ev = RemoteControllerEvent(buf)
             else:
-                ev = DataEvent(id, buf)
+                ev = _DataEvent(id, buf)
             self.__events.append(ev)
 
         while True:
@@ -152,9 +186,9 @@ class Parser:
                 break
 
             if id in range(BYTE, WORD):
-                self.__events.append(ByteEvent(id, self.__r.read(1)))
+                self.__events.append(_ByteEvent(id, self.__r.read(1)))
             elif id in range(WORD, DWORD):
-                self.__events.append(WordEvent(id, self.__r.read(2)))
+                self.__events.append(_WordEvent(id, self.__r.read(2)))
             elif id in range(DWORD, TEXT):
                 add_dwordevent(id, self.__r.read(4))
             else:
@@ -179,7 +213,7 @@ class Parser:
         """Creates and appends `Pattern` objects to `Project`.
         Dispatches `PatternEventID` events to `Pattern` for parsing."""
 
-        if ev.id == Pattern.EventID.New and isinstance(ev, WordEvent):
+        if ev.id == Pattern.EventID.New and isinstance(ev, _WordEvent):
             # Occurs twice, once with the note events only and later again
             # for metadata (name, color and a few undiscovered properties)
             # New patterns can occur for metadata as well; they are empty.
@@ -204,11 +238,6 @@ class Parser:
         if ev.id == Insert.EventID.Output and Insert._count < Insert.max_count:
             self.__proj.inserts.append(self.__cur_ins)
             self.__cur_ins = Insert()
-        elif ev.id == InsertSlot.EventID.DefaultName:
-            if ev.to_str() == "Fruity Wrapper":
-                self.__cur_plug_is_vst = True
-            else:
-                self.__cur_plug_is_vst = False
 
     def __parse_arrangement(self, ev: _EventType):
         """Creates and appends `Arrangement` objects to `Project`. Dispatches
@@ -281,32 +310,19 @@ class Parser:
             )
 
         r = self.__r
-
-        # * Init: Verify header integrity and build event store
-        # Magic number
-        assert r.read(4) == b"FLhd"
-
-        # Header size (always 6)
-        assert r.read_I() == 6
-
-        # Format, TODO enum
-        proj.misc.format = format = r.read_h()
-        assert format == Misc.Format.Song
-
-        # Channel count.
-        # For Patcher presets, the total number of plugins used inside it.
-        proj.misc.channel_count = channel_count = r.read_H()
-        assert channel_count in range(1, 1000)
-
-        # PPQ
-        proj.misc.ppq = Playlist.ppq = r.read_H()
-
-        # Magic number
-        assert r.read(4) == b"FLdt"
-
-        # Combined size of all events
-        _ = r.read_I()
-
+        hdr_magic = r.read(4)
+        if hdr_magic != HEADER_MAGIC:
+            raise InvalidMagicError(hdr_magic)
+        hdr_size = r.read_I()
+        if hdr_size != HEADER_SIZE:
+            raise InvalidHeaderSizeError(hdr_size)
+        proj.misc.format = Misc.Format(r.read_h())
+        proj.misc.channel_count = r.read_H()
+        proj.misc.ppq = _FLObject._ppq = r.read_H()
+        data_magic = r.read(4)
+        if data_magic != DATA_MAGIC:
+            raise InvalidMagicError(data_magic)
+        _ = r.read_I()  # Combined size of all events
         self.__build_event_store()
 
         return self.__events
@@ -319,7 +335,7 @@ class Parser:
 
         # * Modify parsing logic as per FL version
         # TODO: This can be as less as 16. Also insert slots were once 8.
-        Insert.max_count = 127 if _FLObject.fl_version.as_float() >= 12.89 else 104
+        Insert.max_count = 127 if _FLObject._fl_version.as_float() >= 12.89 else 104
         self.__cur_ins = Insert()
 
         # * Build an object model
@@ -375,19 +391,23 @@ class Parser:
         return self.__proj
 
     def parse_zip(self, zip_file, name: str = "") -> Project:
-        """Parses an FLP inside a ZIP and returns a `Project` object.
+        """Parses an FLP inside a ZIP.
 
         Args:
-            zip_file (Union[zipfile.ZipFile, str, bytes, Path, io.BufferedIOBase]): \
-            The path to the ZIP file, stream, Path, file-like or a ZipFile object.
-            name (str, optional): If the ZIP has multiple FLPs, \
-            you need to specify the name of the FLP to parse.
+            zip_file (Union[zipfile.ZipFile, str, bytes, Path, io.BufferedIOBase]):
+                The path to the ZIP file, stream, Path, file-like or a ZipFile
+                object.
+            name (str, optional): If the ZIP has multiple FLPs, you need
+                to specify the name of the FLP to parse.
 
         Raises:
-            TypeError: When `zip_file` points to a ZIP or stream \
-            containing no FLPs.
-            TypeError: When `name` is empty and `zip_file` points \
-            to a ZIP or stream containing more than one FLP.
+            TypeError: When `zip_file` points to a ZIP or stream containing no
+                files with an extension of .flp.
+            TypeError: When `name` is empty and `zip_file` points to a ZIP or
+                stream containing more than one FLP.
+
+        Returns:
+            Project: The parsed object.
         """
 
         flp = None
