@@ -1,8 +1,23 @@
+# PyFLP - An FL Studio project file (.flp) parser
+# Copyright (C) 2022 demberto
+#
+# This program is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option)
+# any later version. This program is distributed in the hope that it will be
+# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
+# Public License for more details. You should have received a copy of the
+# GNU General Public License along with this program. If not, see
+# <https://www.gnu.org/licenses/>.
+
 import enum
 
-from pyflp.event import _DataEventType
-from pyflp.properties import _EnumProperty, _UIntProperty, _UIntValidator
-from pyflp.plugin.plugin import _EffectPlugin
+from pyflp._event import _DataEventType
+from pyflp._properties import _EnumProperty, _UIntProperty
+from pyflp.plugin._plugin import _EffectPlugin
+
+__all__ = ["Soundgoodizer"]
 
 
 class Soundgoodizer(_EffectPlugin):
@@ -32,7 +47,7 @@ class Soundgoodizer(_EffectPlugin):
     mode: Mode = _EnumProperty(Mode)
     """See `Mode`. Default: `Mode.A`"""
 
-    amount: int = _UIntProperty(_UIntValidator(1000))
+    amount: int = _UIntProperty(max_=1000)
     """Amount. Min: 0, Max: 1000, Default: 600. Logarithmic."""
 
     def _parse_data_event(self, e: _DataEventType) -> None:
